package uk.ac.cf.spring.demo.sports.rankingtable;

import lombok.Getter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
@Repository

public class RankingRepository {
    public List<RankingTable> findBySportAndOrder(String sport, String order) {
        // 确保 order 参数是有效值
        if (!"asc".equalsIgnoreCase(order) && !"desc".equalsIgnoreCase(order)) {
            order = "desc";
        }

        String sql = "SELECT * FROM ranking WHERE sport = ? ORDER BY wins " + order.toUpperCase();
        return jdbcTemplate.query(sql, mapRowToRanking(), sport);
    }


    @Getter
    public static class MatchResult {
        private final Long winnerId;
        private final String sport;

        public MatchResult(Long winnerId, String sport) {
            this.winnerId = winnerId;
            this.sport = sport;
        }

    }

    private final JdbcTemplate jdbcTemplate;


    public RankingRepository(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    // get all rankings sql code
    public List<RankingTable> findAll() {
        String sql = "SELECT * FROM ranking";
        return jdbcTemplate.query(sql, mapRowToRanking());
    }

    public List<RankingTable> findBySport(String sport) {
        System.out.println(sport);
        String sql = "SELECT * FROM ranking WHERE sport = ?";
        return jdbcTemplate.query(sql, mapRowToRanking(), sport);
    }


    // get ranking by userid and sport
    public RankingTable findByUserIdAndSport(Long userId, String sport) {
        String sql = "SELECT * FROM ranking WHERE user_id = ? AND sport = ?";
        return jdbcTemplate.queryForObject(sql, mapRowToRanking(), userId, sport);
    }

    // update rankings
    public void saveOrUpdate(Long userId, String sport, int newWins) {
        String sql = """
            INSERT INTO ranking (user_id, sport, wins)
            VALUES (?, ?, ?)
            ON DUPLICATE KEY UPDATE wins = ?
        """;
        jdbcTemplate.update(sql, userId, sport, newWins, newWins);
    }


    // get result from match where status is completed
    public List<MatchResult> findWinnersAndSportsFromCompletedMatches() {
        String sql = """
            SELECT winner_id, sport
            FROM match_item
            WHERE status = 'completed'
        """;
        return jdbcTemplate.query(sql, (rs, rowNum) -> new MatchResult(
                rs.getLong("winnerId"),
                rs.getString("sport")

        ));
    }
    public List<RankingTable> findAllSortedByWins(String order) {
        // 确保order参数的值是"asc"或"desc"，如果不是则默认为"desc"
        if (order == null || (!order.equalsIgnoreCase("asc") && !order.equalsIgnoreCase("desc"))) {
            order = "asc";  // 默认降序
        }

        String sql = "SELECT * FROM ranking ORDER BY wins " + order.toUpperCase();
        return jdbcTemplate.query(sql, mapRowToRanking());
    }
    //总胜利数查询
    public List<TotalWinsDTO> findTotalWinsByAll(String order) {
        // 验证 order 参数是否合法，只允许 "asc" 或 "desc"
        String sortOrder = "desc"; // 默认值
        if ("asc".equalsIgnoreCase(order)) {
            sortOrder = "asc";
        }

        String sql = "SELECT user_id,username,'totalwins' AS sport, SUM(wins) AS wins FROM ranking GROUP BY user_id ORDER BY wins "  + sortOrder;


        return jdbcTemplate.query(sql, mapRowToTotalWinsDTO());
    }

    // a function can merge result into a ranking
    private RowMapper<RankingTable> mapRowToRanking() {
        return (rs, rowNum) -> {
            RankingTable ranking = new RankingTable();
            ranking.setId(rs.getLong("id"));  // 映射 id
            ranking.setUserId(rs.getLong("user_id"));  // 映射 user_id
            ranking.setSport(rs.getString("sport"));  // 映射 sport
            ranking.setWins(rs.getInt("wins"));  // 映射 wins
            ranking.setUsername(rs.getString("username"));  // 映射 username
            return ranking;
        };
    }
    //聚合查询专用映射方法
    private RowMapper<TotalWinsDTO> mapRowToTotalWinsDTO() {
        return (rs, rowNum) -> {
            TotalWinsDTO dto = new TotalWinsDTO();
            dto.setUsername(rs.getString("username"));
            dto.setUserId(rs.getLong("user_id"));
            dto.setSport("totalwins"); // 聚合查询时 sport 固定为 "totalwins"
            dto.setWins(rs.getInt("wins"));
            return dto;
        };
    }

}

